/* This file was generated by syncqt. */
#ifndef QT_QTQUICKWIDGETS_VERSION_H
#define QT_QTQUICKWIDGETS_VERSION_H

#define QTQUICKWIDGETS_VERSION_STR "5.15.18"

#define QTQUICKWIDGETS_VERSION 0x050F12

#endif // QT_QTQUICKWIDGETS_VERSION_H
